
(cc Travel)

(defconcept Travel-Agency
  :is-primitive Enterprise)

(defconcept Client
  :is-primitive Natural-Person)

(defconcept Mean-Of-Transport
  :is-primitive Instrument
  :partitions $Transport$)

(defconcept Instrument
  :is-primitive Artifact)

(defconcept Accommodation
  :is-primitive Living-Quarters
  :implies (:filled-by HAS-QUALITY Public))

(defconcept Living-Quarters
  :is-primitive Housing)
 ;;; :implies (:some HOSTS Person))

(defconcept Housing
  :is-primitive Structure)

(defconcept Structure
  :is-primitive Artifact)

(defconcept Plane
  :is-primitive Mean-Of-Transport
  :in-partition $Transport$)

(defconcept Train
  :is-primitive Mean-Of-Transport
  :in-partition $Transport$)

(defconcept Car
  :is-primitive Mean-Of-Transport
  :in-partition $Transport$)

(defconcept Ferry
  :is-primitive Mean-Of-Transport
  :in-partition $Transport$)

(defconcept Motorbike
  :is-primitive Mean-Of-Transport
  :in-partition $Transport$)

(defconcept Bike
  :is-primitive Mean-Of-Transport
  :in-partition $Transport$)

(defconcept Ship
  :is-primitive Mean-Of-Transport
  :in-partition $Transport$)

(defconcept Underground
  :is-primitive Mean-Of-Transport
  :in-partition $Transport$)

(defconcept City-Bus
  :is-primitive Mean-Of-Transport
  :in-partition $Transport$)

(defconcept Taxi
  :is-primitive Car)

(defconcept Rental-Car
  :is-primitive Car)

(defconcept City-Transport
  :is (:or Underground
           City-Bus
           Taxi
           Rental-Car))

(defconcept Flight-Company
  :is-primitive Enterprise)

(defconcept Boeing-Plane
  :is (:and Plane
            (:filled-by OWNED-BY Boeing)))

(defconcept Airbus-Plane
  :is (:and Plane
            (:filled-by OWNED-BY Airbus)))

(defconcept Boeing-717
  :is-primitive Boeing-Plane)

(defconcept Boeing-777
  :is-primitive Boeing-Plane)

(defconcept Flight   ;;;flight as a person's travel
  :is (:and Travel
            (:some AGENT Person)
            (:some INSTRUMENT Plane)
            (:some PROVIDER Flight-Company)
            (:exactly 1 DEPARTURE-DATE)
            (:exactly 1 DEPARTURE-TIME)
            (:exactly 1 FLIGHT-DEPARTURE-CITY)
            (:exactly 1 DEPARTURE-AIRPORT)
            (:exactly 1 ARRIVAL-DATE)
            (:exactly 1 ARRIVAL-TIME)
            (:exactly 1 FLIGHT-ARRIVAL-CITY)
            (:exactly 1 ARRIVAL-AIRPORT)
            (:at-least 1 PRICE))
  :implies (:and (:some PART-OF Travel)
                 (:some SPECIFICALLY-CONSTANTLY-DEPENDENT-ON Flight-Service)))

(defconcept Flight-Service   ;;;flight as a provider's service
  :is (:and Service
            (:some AGENT Flight-Company)
            (:some PATIENT Person)
            (:some INSTRUMENT Plane)
            (:at-least 1 DEPARTURE-DATE)
            (:at-least 1 DEPARTURE-TIME)
            (:exactly 1 FLIGHT-DEPARTURE-CITY)
            (:exactly 1 DEPARTURE-AIRPORT)
            (:at-least 1 ARRIVAL-DATE)
            (:at-least 1 ARRIVAL-TIME)
            (:exactly 1 FLIGHT-ARRIVAL-CITY)
            (:exactly 1 ARRIVAL-AIRPORT)
            (:at-least 1 PRICE)))            

(defconcept Travel
  :is-primitive Activity
  :implies (:and (:some HAS-LOCATION-WITHIN Geographic-Area)
                 (:some INSTRUMENT Mean-Of-Transport)
                 (:some ORIGIN Geographic-Area)
                 (:some DESTINATION Geographic-Area)
                 (:some TRAVEL-DISTANCE Number)
                 (:at-least 1 PART)))

(defconcept Overnight-Travel
  :is (:and Travel
            (:some PART Lodging)))

(defconcept City-Travel
  :is (:and Travel
            (:exactly 1 HAS-LOCATION-WITHIN City))
  :implies (:some INSTRUMENT City-Transport))

(defconcept Departure
  :is-primitive (:and Activity
                      (:some PART-OF Travel)
                      (:some HAS-LOCATION-WITHIN Geographic-Area)))

(defconcept Arrival
  :is-primitive (:and Activity
                      (:some PART-OF Travel)
                      (:some HAS-LOCATION-WITHIN Geographic-Area)))

(defconcept Distance
  :is-primitive Physical-Quality
  :implies (:and (:all INHERES-IN Geographic-Area)
                 (:at-least 1 INHERES-IN 
                            (:and Geographic-Area
                                  (:exactly 2 SPATIAL-BOUNDARY Geographic-Area)))
                 (:some HAS-QUALE (:the NUMEROSITY Number))
                 (:some UNIT Measurement-Unit)))

(defconcept One-Way-Flight
  :is (:and Flight
            (:exactly 0 PART-OF 
                      (:and Travel
                            (:some PART Return-Flight)))))

(defconcept Round-Trip-Flight
  :is (:and Flight
            (:at-least 2 |Member| Flight)
            (:exactly 1 PART Outward-Flight)
            (:exactly 1 PART Return-Flight)
            (:at-least 2 PART Departure)
            (:at-least 2 PART Arrival)))

(defconcept Outward-Flight
  :is-primitive Flight
  :implies (:and (:some PART-OF Round-Trip-Flight)
                 (:some PART Departure)
                 (:some PART Arrival)
                 (:some PRECEDES Return-Flight)))

(defconcept Return-Flight
  :is-primitive Flight
  :implies (:and (:some PART-OF Round-Trip-Flight)
                 (:some PART Departure)
                 (:some PART Arrival)
                 (:some FOLLOWS Outward-Flight)))

(defconcept Europe-America-Travel
  :is (:and Travel
            (:the ORIGIN (:one-of Europe America))
            (:the DESTINATION (:one-of Europe America))
            (:all INSTRUMENT (:or Plane Ship))
            (:some INSTRUMENT (:or Plane Ship))))

(defconcept Date-Value
  :is-primitive Temporal-Value)

(defconcept Time-Value
  :is-primitive Temporal-Value)

(defconcept Metric-Value
  :is-primitive Spatial-Value)

(defconcept Price-Value
  :is-primitive Abstract-Value)

(defconcept First-Class-Price-Value
  :is-primitive Price-Value)

(defconcept Business-Class-Price-Value
  :is-primitive Price-Value)

(defconcept Economy-Class-Price-Value
  :is-primitive Price-Value)

(defconcept City
  :is-primitive Geographic-Area
  :implies (:some PART-OF State))

(defconcept Resort
  :is-primitive Geographic-Area
  :implies (:some PART-OF City))

(defconcept Nation
  :is-primitive Geographic-Area
  :implies (:some PART-OF Continent))

(defconcept Continent
  :is-primitive Geographic-Area)  

(defconcept Airport
  :is-primitive Facility
  :implies (:some CLOSE-PLACE City))

(defconcept Facility
  :is-primitive Artifact)

(defconcept Hotel
  :is-primitive Accommodation
  :implies (:and (:some HAS-ADDRESS Address)
                 (:some HAS-PHONE-NUMBER Phone-Number)
                 (:some HAS-URL Url)
                 (:some HAS-PERSON-CAPACITY Number)
                 (:some COMPONENT Room)
                 (:some HAS-ROOM-CAPACITY Number)
                 (:some HAS-ROOM-AVAILABILITY Number)
                 (:some DESCRIPTION String)
                 (:exactly 1 DOG-ALLOWANCE Boolean))
  :partitions $Hotels$)

(defconcept Hotel-Chain
  :is-primitive Enterprise)

(defconcept Enteprise
  :is-primitive Organization)

(defconcept B&B
  :is-primitive Accommodation)

(defconcept One-Star-Hotel
  :is (:and Hotel
            (:filled-by HAS-QUALITY One-Star))
  :in-partition $Hotels$)

(defconcept Two-Star-Hotel
  :is (:and Hotel
            (:filled-by HAS-QUALITY Two-Star))
  :in-partition $Hotels$)

(defconcept Three-Star-Hotel
  :is (:and Hotel
            (:filled-by HAS-QUALITY Three-Star))
  :in-partition $Hotels$)

(defconcept Four-Star-Hotel
  :is (:and Hotel
            (:filled-by HAS-QUALITY Four-Star))
  :in-partition $Hotels$)

(defconcept Five-Star-Hotel
  :is (:and Hotel
            (:filled-by HAS-QUALITY Five-Star))
  :in-partition $Hotels$)

(defconcept Accommodation-Quality
  :is-primitive Abstract-Quality)

(defconcept Beach
  :is-primitive Geographic-Area)

(defconcept Skiing-Area
  :is-primitive Geographic-Area)

(defconcept Room
  :is-primitive (:and Structure
                      (:some PART-OF Housing)))

(defconcept Hotel-Room
  :is (:and Room
            (:some COMPONENT-OF Hotel)
            (:some BED-NUMBER Number)
            (:some USE-PRICE Number)
            (:some COMPONENT Instrument))
  :implies (:some USED-IN Hosting))

(defconcept Hosting
  :is-primitive Service
  :implies (:and (:some RESOURCE Accommodation)
                 (:some PATIENT Natural-Person)))

(defconcept Lodging
  :is-primitive Activity
  :implies (:and (:some AGENT Natural-Person)
                 (:some RESOURCE Accommodation)
                 (:some SPECIFICALLY-CONSTANTLY-DEPENDENT-ON Hosting)))

(defconcept Service
  :is-primitive Work)

(defconcept Work
  :is-primitive Activity)

(defconcept Bed
  :is-primitive Furniture)

(defconcept Furniture
  :is-primitive Instrument)

(defconcept Internet-Connection
  :is-primitive Instrument)

(defconcept TV
  :is-primitive Receiver)

(defconcept Receiver
  :is-primitive Electronic-Equipment)

(defconcept Electronic-Equipment
  :is-primitive Equipment)

(defconcept Equipment
  :is-primitive Instrument)

(defconcept Service-Provider
  :is (:and Social-Object
            (:some PERFORMS Service)))

(defconcept Request
  :is-primitive Activity)

(defconcept Hate
  :is-primitive Emotion)

(defconcept Address
  :is-primitive Spatial-Location)

(defconcept Url
  :is-primitive Abstract-Location)

(defconcept Phone-Number
  :is-primitive String)


;;;*******
(defrelation DATE
  :is (:and MIXING-RELATION
            (:compose (:and HAS-TEMPORAL-LOCATION
                            (:domain Occurrence))
                      (:and HAS-QUALE
                            (:range Date-Value)))))
(defrelation DATE-OF :is (:inverse DATE))

(defrelation TIME
  :is (:and MIXING-RELATION
            (:compose (:and HAS-TEMPORAL-LOCATION
                            (:domain Occurrence))
                      (:and HAS-QUALE
                            (:range Time-Value)))))
(defrelation TIME-OF :is (:inverse TIME))

(defrelation HAS-ADDRESS
  :is-primitive HAS-SPATIAL-LOCATION
  :range Address)

(defrelation HAS-URL
  :is-primitive HAS-ABSTRACT-LOCATION
  :range Url)

(defrelation HAS-LOCATION-WITHIN
  :is (:and MIXING-RELATION
            (:compose PART-OF
                      HAS-SPATIAL-LOCATION)))
(defrelation COMPLETE-LOCATION-OF
  :is (:inverse HAS-LOCATION-WITHIN))
                
                

(defrelation HAS-PHONE-NUMBER
  :is-primitive EXTRINSIC-RELATION
  :range Phone-Number)

(defrelation PRICE
  :is (:and HAS-VALUE
            (:composition HAS-QUALITY
                          (:and HAS-QUALE (:range Price-Value))
                          HAS-QUALE)
            (:domain Service)
            (:range Number)))

(defrelation USE-PRICE
  :is (:and MIXING-RELATION
            (:composition USED-IN PRICE))
  :domain Object)

(defrelation HAS-VALUE
  :is (:and EXTRINSIC-RELATION
            (:composition HAS-QUALITY HAS-QUALE NUMEROSITY)
            (:range Number)))

(defrelation CLOSE-PLACE
  :is-primitive (:and MIXING-RELATION
                      (:compose (:and HAS-LOCATION-WITHIN (:range Geographic-Area))
                                (:and COMPLETE-LOCATION-OF (:range City)))))
(defrelation CLOSE-PLACE-OF
  :is (:inverse CLOSE-PLACE))

(defrelation DEPARTURE-DATE
  :is (:and MIXING-RELATION
            (:composition (:and PART (:domain Travel) (:range Departure))
                          DATE))
  :characteristics :single-valued)

(defrelation DEPARTURE-TIME
  :is (:and MIXING-RELATION
            (:composition (:and PART (:domain Travel) (:range Departure))
                          TIME))
  :characteristics :single-valued)

(defrelation DEPARTURE-AIRPORT
  :is (:and MIXING-RELATION
            (:composition FLIGHT-DEPARTURE-CITY
                          (:and CLOSE-PLACE-OF (:range Airport))))
  :characteristics :single-valued)

(defrelation FLIGHT-DEPARTURE-CITY
  :is-primitive ORIGIN
  :range City
  :characteristics :single-valued)

(defrelation ARRIVAL-DATE
  :is (:and MIXING-RELATION
            (:composition (:and PART (:domain Travel) (:range Arrival))
                          DATE))
  :characteristics :single-valued)

(defrelation ARRIVAL-TIME
  :is (:and MIXING-RELATION
            (:composition (:and PART (:domain Travel) (:range Arrival))
                          TIME))
  :characteristics :single-valued)

(defrelation ARRIVAL-AIRPORT
  :is (:and MIXING-RELATION
            (:composition FLIGHT-ARRIVAL-CITY
                          (:and CLOSE-PLACE-OF (:range Airport))))
  :characteristics :single-valued)

(defrelation FLIGHT-ARRIVAL-CITY
  :is-primitive DESTINATION
  :range City
  :characteristics :single-valued)

(defrelation ORIGIN
  :is (:and MIXING-RELATION
            (:composition (:and PART (:range Departure))
                          HAS-LOCATION-WITHIN)
            (:domain Travel)
            (:range Geographic-Area)))
  ;:characteristics :single-valued)

(defrelation DESTINATION
  :is (:and MIXING-RELATION
            (:composition (:and PART (:range Arrival))
                          HAS-LOCATION-WITHIN)
            (:domain Travel)
            (:range Geographic-Area)))
  ;:characteristics :single-valued)

(defrelation LODGES-IN
  :is (:and MIXING-RELATION
            (:composition (:and HAS-LOCATION-WITHIN (:range City))
                          (:and COMPLETE-LOCATION-OF (:range Accommodation)))))

(defrelation TRAVEL-DISTANCE
  :is (:and EXTRINSIC-RELATION
            (:composition (:and HAS-SPATIAL-LOCATION 
                                (:domain Travel)
                                (:range Geographic-Area))
                          (:and HAS-QUALITY (:range Distance))
                          HAS-QUALE 
                          NUMEROSITY))
  :range Number)

(defrelation HAS-ROOM-CAPACITY
  :is (:and EXTRINSIC-RELATION
            (:composition (:and COMPONENT (:range Room))
                          HAS-VALUE))
  :characteristics :single-valued)

(defrelation HAS-ROOM-AVAILABILITY
  :is (:and EXTRINSIC-RELATION
            (:composition (:and COMPONENT (:range Room))
                          HAS-VALUE))
  :characteristics :single-valued)

(defrelation BED-NUMBER
  :is (:and EXTRINSIC-RELATION
            (:composition (:and COMPONENT (:range Bed))
                          HAS-VALUE))
  :characteristics :single-valued)

(defrelation HAS-PERSON-CAPACITY
  :is (:and EXTRINSIC-RELATION
            (:composition (:and HOSTS (:range Natural-Person))
                          HAS-VALUE))
  :characteristics :single-valued)

(defrelation HOSTS
  :is (:and MIXING-RELATION
            (:composition (:and RESOURCE-FOR (:range Hosting))
                          (:and PATIENT (:range Natural-Person)))))

(defrelation DESCRIPTION
  :is-primitive DOCUMENTATION)

(defrelation DOG-ALLOWANCE
  :is-primitive BOOLEAN-QUALITY
  :range Boolean
  :characteristics :single-valued)

(defrelation BOOLEAN-QUALITY
  :is (:and HAS-QUALITY
            (:range Boolean)))

(defrelation BEACH-DISTANCE
  :is-primitive (:and EXTRINSIC-RELATION
                      (:composition HAS-QUALITY
                                    (:and HAS-QUALE (:range Metric-Value))
                                    HAS-QUALE
                                    NUMEROSITY))
  :range Number)
(defrelation BEACH-DISTANCE-OF
  :is (:inverse BEACH-DISTANCE))

(defrelation SKIING-DISTANCE
  :is-primitive (:and EXTRINSIC-RELATION
                      (:composition HAS-QUALITY
                                    (:and HAS-QUALE (:range Metric-Value))
                                    HAS-QUALE
                                    NUMEROSITY))
  :range Number)
(defrelation SKIING-DISTANCE-OF
  :is (:inverse SKIING-DISTANCE))

(defrelation HATES
  :is (:and MIXING-RELATION
            (:composition (:and EXPERIENCES (:range Hate))
                          (:and TARGET (:range Entity)))))
(defrelation HATED-BY
  :is (:inverse HATES))


(defrelation REQUESTS
  :is (:and MIXING-RELATION
            (:composition (:and PERFORMS (:range Request))
                          (:and METHOD-OF (:range Service))
                          (:and PERFORMED-BY (:range Service-Provider)))))

(defrelation PROVIDER
  :is-primitive PERFORMED-BY
  :domain Service
  :range Social-Object)
(defrelation PROVIDER-OF
  :is (:inverse PROVIDER))
